import java.io.*;
import java.net.*;

/**
 * 7DS mobile node which listens to the multicast advertisements of a
 * nearby 7DS access point. Once an access point is found, it'll try 
 * to access the server port and establish a socket connection.
 */
public class Client {
  public static void main(String[] args) {
    /* Subscribe to a multicast group and listen for service advertisements. */
    String addr = null;
    try {
      MulticastSocket adSocket = new MulticastSocket(Server.MULTICAST_PORT);
      InetAddress multicastGroup = InetAddress.getByName(Server.MULTICAST_ADDR);
      adSocket.joinGroup(multicastGroup);

      byte[] buf = new byte[256];
      DatagramPacket packet = new DatagramPacket(buf, buf.length);
      adSocket.receive(packet);
      addr = new String(packet.getData()).trim();
      System.out.println("Server " + addr + " is found.");
      adSocket.leaveGroup(multicastGroup);
      adSocket.close();
    } catch (Exception e) {
      System.err.println("Cannot subscribe to multicast address "+
          Server.MULTICAST_ADDR + ":" + Integer.toString(Server.MULTICAST_PORT));
      System.exit(1);
    }

    /* Once a 7DS access point is found, establish a socket connection with the
       server and start the communication process. */
    Socket clientSocket = null;
    PrintWriter out = null;
    BufferedReader in = null;
    try {
      InetAddress serverAddress = InetAddress.getByName(addr);
      clientSocket = new Socket(serverAddress, Server.SERVER_PORT);
      in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
      System.out.println("Server: " + in.readLine());

      out = new PrintWriter(clientSocket.getOutputStream(), true);
      BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));
      String fromUser;
      while (true) {
        fromUser = stdIn.readLine();
	    if (fromUser != null) {
          System.out.println("Client: " + fromUser);
          out.println(fromUser);
          if (fromUser.equals("Bye")) break;
	    }
      }
      in.close();
      out.close();
      stdIn.close();
      clientSocket.close();
      
    } catch (UnknownHostException e) {
      System.err.println("Unknown host: " + addr);
      System.exit(1);
    } catch (IOException e) {
      System.err.println("Cannot get I/O for the connection to: " + addr);
      System.exit(1);
    }
  }
}
