import java.io.*;
import java.net.*;

/**
 * 7DS Server which periodically multicasts its IP address to nearby
 * mobile nodes. A mobile node responds by sending a request to the
 * service gateway. The server then creates a new thread and a new
 * socket connection for the mobile node.
 */
public class Server {
  public static int SERVER_PORT = 8000;
  public static int MULTICAST_PORT = 8000;
  public static String MULTICAST_ADDR = "224.0.0.1";

  public Server() {
    new Thread(new ServiceAdvertisement()).start();
    new Thread(new ServiceGateway()).start();
  }

  public static void main(String[] args) {
    Server s = new Server();
  }
  
  /**
   * Implementation of a thread which multicasts periodic service
   * advertisements.
   */
  class ServiceAdvertisement implements Runnable {
    public void run() {
      try {
        DatagramSocket registrarSocket = new DatagramSocket();
        InetAddress multicastGroup = InetAddress.getByName(MULTICAST_ADDR);
        String adString = InetAddress.getLocalHost().getHostAddress();
        byte[] adBuffer = adString.getBytes();
        DatagramPacket adPacket =
            new DatagramPacket(adBuffer, adBuffer.length, multicastGroup, MULTICAST_PORT);
        while (true) {
            registrarSocket.send(adPacket);
            Thread.sleep(1000);
        }
      } catch (Exception e) {
      }
    }
  }
  
  /**
   * Implementation of a gateway that accepts the requests from nearby
   * mobile nodes and responds by creating a new thread and a new
   * socket connection for each client.
   */
  class ServiceGateway implements Runnable {
    public void run() {
      ServerSocket serverSocket = null;
      boolean listening = true;
      try {
        serverSocket = new ServerSocket(SERVER_PORT);
      } catch (IOException e) {
        System.err.println("Could not listen on port: 8000.");
        System.exit(1);
      }

      while (listening) {
        try {
          new Thread(new Service(serverSocket.accept())).start();
          Thread.sleep(1000);
        } catch (IOException e) {
          System.err.println("Accept failed.");
          System.exit(1);
        } catch (InterruptedException e) {
        }
      }
    }
  }

  /**
   * Implementation of a thread that provides service to a client.
   */
  class Service implements Runnable {
    private Socket clientSocket = null;

    public Service(Socket clientSocket) {
      this.clientSocket = clientSocket;
    }

    public void run() {
	  try {
	    PrintWriter out = new PrintWriter(clientSocket.getOutputStream(), true);
	    BufferedReader in = new BufferedReader(
				    new InputStreamReader(
				    clientSocket.getInputStream()));
        out.println("You are successfully connected to " +
            clientSocket.getLocalAddress().toString() + ":" +
            Integer.toString(clientSocket.getLocalPort()));
        String inputLine = null;
        while (true) {
          inputLine = in.readLine();
          if (inputLine != null) {
            if (!inputLine.equals("Bye")) {
              System.out.println(inputLine);
            } else
              break;
          }
        }
        out.println("Bye");
	    out.close();
	    in.close();
	    clientSocket.close();
	  } catch (IOException e) {
	    e.printStackTrace();
	  }
    }
  }
}
